(function() {
  "use strict";

  var Categories = require.main.require("./src/categories");
  var db = require.main.require("./src/database");

  exports.filterTopicsPrepare = function(data, callback) {
    var sets = typeof data.set === "string" ? [data.set] : data.set;
    getCategoryIDs(data.data.cid, function(err, retEl) {
      if (retEl.type == 'cid' && retEl.children.length > 0) {
        retEl.children.forEach(function(child) {
        	sets.push("cid:" + child + ":tids");
        });
      }else if (retEl.type == 'tag' && retEl.children.length > 0){
	sets = [];
      	retEl.children.forEach(function(child) {
		sets.push("tag:" + child + ":topics");
	});
      }
      data.set = [{ unionArray: sets }];
      return callback(null, data);
    });
  };

  exports.init = function init(params, callback) {
    var orig_getSortedSetRevIntersect = db.getSortedSetRevIntersect;
    db.getSortedSetRevIntersect = function(params, callback) {
      var sets = params.sets;
      if (Array.isArray(sets) && sets[0] && sets[0].unionArray) {
        return db.getSortedSetRevRange(
          sets[0].unionArray,
          params.start,
          params.stop,
          callback
        );
      } else {
        return orig_getSortedSetRevIntersect.apply(this, arguments);
      }
    };

    var orig_getSortedSetIntersect = db.getSortedSetIntersect;
    db.getSortedSetIntersect = function(params, callback) {
      var sets = params.sets;
      if (Array.isArray(sets) && sets[0] && sets[0].unionArray) {
        return db.getSortedSetRange(
          sets[0].unionArray,
          params.start,
          params.stop,
          callback
        );
      } else {
        return orig_getSortedSetIntersect.apply(this, arguments);
      }
    };

    return callback();
  };

  function getCategoryIDs(cid, resolve) {
    	var dbClient = db.client; // get the database client
	db.pool.query({
		text: "SELECT name,category_type FROM zrc_tags WHERE nodebb_cat_id = $1::numeric",
		values: [cid]
	},function(err, res){
		if(err){
			console.log(err);
			resolve(null,[]);
		}
		if(res.rowCount > 0){
			var curRow = res.rows[0];
			if(curRow.category_type == 'SC'){
				var tagArray = [];
				res.rows.forEach(function(value,i){
					tagArray.push(value.name);
				});
				resolve(null,{children:tagArray,type: 'tag'});
			}else{
				db.pool.query({
					text: "SELECT data->>'cid' as cid FROM legacy_hash where data->>'parentCid'=$1::text and _key like 'category:%'",       
					values: [cid.toString()]
    				}, function(err, res2) {
					if (err) {
	    					return handleError(err, callback, function() {
							resolve(null, []);
	    					});
					}
					var cidArray = [];
					res2.rows.forEach(function(value,i){
						cidArray.push(parseInt(value.cid));
					});
					resolve(null,{children:cidArray,type: 'cid'});
    				});
			}
		}else{
			resolve(null,[]);
		}
	});
  }
})();
